import Alpaca from '@alpacahq/alpaca-trade-api';
import { ActionSide, ActionFetchOptions, ActionFetchResponse, ActionDateType } from '../interface/actions';

/**
 * The activity object returned by Alpaca.
 * @see https://docs.alpaca.markets/reference/getaccountactivitiesbyactivitytype
 */
interface AlpacaActivity {
    id: string;
    activity_type: string;
    transaction_time: string;
    type: string;
    price: string;
    qty: string;
    side: string;
    symbol: string;
    leaves_qty: string;
    order_id: string;
    cum_qty: string;
    order_status: string;
}

export class AlpacaActionProvider {
    readonly alpaca: Alpaca;

    readonly fetchActions = (options: ActionFetchOptions): Promise<ActionFetchResponse> => {
        return (this.alpaca.getAccountActivities({
            activityTypes: "FILL",
            until: options.dateOptions?.dateType === ActionDateType.Before ? options.dateOptions.date : undefined,
            after: options.dateOptions?.dateType === ActionDateType.After ? options.dateOptions.date : undefined,
            direction: "desc",
            date: options.dateOptions?.dateType === ActionDateType.On ? options.dateOptions.date : undefined,
            pageSize: options.pageSize,
            pageToken: undefined,
        }) as Promise<AlpacaActivity[]>).then((activities) => {
            return {
                actions: activities
                .filter((activity) => activity.order_status === "filled")
                .map((activity) => {
                    return {
                        symbol: activity.symbol,
                        quantity: parseInt(activity.qty, 10),
                        side: activity.side === "buy" ? ActionSide.Buy : ActionSide.Sell,
                        pricePerShare: parseFloat(activity.price),
                        timestamp: new Date(activity.transaction_time),
                    };
                }),
                fetchNextPage: undefined
            };
        }).catch((err) => {
            return err;
        });
    };

    constructor(alpaca: Alpaca) {
        this.alpaca = alpaca;
    }
}
